@@ -3,6 +3,8 @@ from __future__ import division
 import tensorflow as tf
 import numpy as np
 
+from onnx_tf.common import get_data_format
+from onnx_tf.common import get_perm_from_formats
 from onnx_tf.common import pooling_helper
 from onnx_tf.common.tf_helper import tf_shape
 from onnx_tf.common.tf_helper import tf_product
@@ -181,6 +183,10 @@ class DilatedPooling(object):
     else:
       self.padding_constant = 0
 
+    self.storage_format, self.compute_format = get_data_format(
+        self.spatial_size + 2)
+    self.need_trans = self.storage_format != self.compute_format
+
   def _calc_input_ind(self, output_ind, kernel, dilation, stride):
     """
             This function maps index from the output of _remove_dilations
@@ -228,11 +234,11 @@ class DilatedPooling(object):
 
             Maps indices generated by maxpool_with_argmax on top of the
             dilation reduced input to the orignal input indices
-        """
+    """
 
-    in_width = self.orig_input_shape[2]
-    num_channels = self.orig_input_shape[3]
-    output_width = self.output_shape[2]
+    in_width = self.orig_input_shape[3]
+    num_channels = self.orig_input_shape[1]
+    output_width = self.output_shape[3]
 
     # mod_floor op is not implemented on GPU
     # implement it using: a % b = a - (a // b) * b
@@ -284,18 +290,12 @@ class DilatedPooling(object):
               the result is:
                          [[ 10, 11],
                           [ 14, 15]]
-        """
+    """
 
     input_shape = tf_shape(self.input)
-    in_spatial_shape = input_shape[1:self.spatial_size + 1]
+    in_spatial_shape = input_shape[2:]
 
-    channels_count = input_shape[self.spatial_size + 1]
-    # Initialize gather_ind with the range of channels
-    # e.g. [0 1]
-    gather_ind = tf.range(channels_count, dtype=tf.int64)
-    # convert the vector to column vector
-    # in the following logic we use column vectors
-    gather_ind = tf.expand_dims(gather_ind, 1)
+    channels_count = input_shape[1]
 
     # initilize the output_shape with zeros
     # self.output_shape will contain the shape of the
@@ -366,15 +366,15 @@ class DilatedPooling(object):
 
             These are the indices used for gather_nd operation to collect
             the values from the input data.
-        """
+    """
 
     for dim in range(self.spatial_size - 1, -1, -1):
       filter_size = (self.kernel_shape[dim] - 1) * \
                      self.dilations[dim] + 1
       output_size = ((
-          (in_spatial_shape[dim] - filter_size) // self.strides[dim]) + 1
-                    ) * self.kernel_shape[dim]
-      self.output_shape[dim + 1] = output_size
+          (in_spatial_shape[dim] - filter_size) // self.strides[dim]) +
+                     1) * self.kernel_shape[dim]
+      self.output_shape[dim + 2] = output_size
 
       # initialize the output dimension index with the range of the
       # dimension output size (e.g. 4): [0, 1, 2, 3]
@@ -388,16 +388,25 @@ class DilatedPooling(object):
       # convert to column vector
       dim_ind = tf.expand_dims(dim_ind, 1)
 
-      # "combine" current dimension indices with the previous dimensions
-      # using cartesian product
-      gather_ind = tf_product(dim_ind, gather_ind)
+      if (dim == self.spatial_size - 1):
+        gather_ind = dim_ind
+      else:
+        # "combine" current dimension indices with the previous dimensions
+        # using cartesian product
+        gather_ind = tf_product(dim_ind, gather_ind)
 
     # The result from the above loop for 2D data will be:
-    # [[y1, x1, c], [y2, x2, c], ..., [yn, xm, c]] where n is the height,
-    # m is the width and c is the channel number.
+    # [[y1, x1], [y2, x2], ..., [yn, xm]] where n is the height,
+    # m is the width.
 
     # set the channels count in the output_shape
-    self.output_shape[self.spatial_size + 1] = channels_count
+    self.output_shape[1] = channels_count
+    # create the channel indices
+    channel_ind = tf.range(channels_count, dtype=tf.int64)
+    # convert to column vector
+    channel_ind = tf.expand_dims(channel_ind, 1)
+    # "combine" channel indices with the result from the loop
+    gather_ind = tf_product(channel_ind, gather_ind)
 
     # expand the dimensions to match the input dimensions + 1
     for x in range(self.spatial_size):
@@ -416,7 +425,7 @@ class DilatedPooling(object):
   def _calc_pads_same(self, in_spatial_shape):
     """
             Calculate SAME_* paddings.
-        """
+    """
 
     pad_ops = pooling_helper.pad_numpy_ops if self.is_known_shape else \
         pooling_helper.pad_tf_ops
@@ -428,7 +437,7 @@ class DilatedPooling(object):
   def _calc_pads_explicit(self):
     """
             Calculate explicit padding
-        """
+    """
     assert type(self.padding) is list
 
     pads = []
@@ -439,7 +448,7 @@ class DilatedPooling(object):
   def _calc_pads_ceil_mode(self, in_spatial_shape):
     """
             Calculate padding in ceil_mode
-        """
+    """
 
     pads = []
     for i in range(self.spatial_size):
@@ -481,7 +490,7 @@ class DilatedPooling(object):
   def _pad_input(self):
     """
             Pad the input according to the parameters
-        """
+    """
     # check if we need to do any padding at all
     if not self.ceil_mode and ((type(self.padding) is list and
                                 self.padding == [0] * self.spatial_size * 2) or
@@ -489,23 +498,23 @@ class DilatedPooling(object):
       self.pads = np.array([0] * self.spatial_size * 2)
       return (self.input, self.pads)
 
-    in_spatial_shape = self.input_shape[1:self.spatial_size + 1]
+    in_spatial_shape = self.input_shape[2:]
     pads = self._calc_pads(in_spatial_shape)
 
     if self.is_known_shape and np.count_nonzero(pads) == 0:
       self.pads = pads
       return (self.input, pads)
 
-    tf_paddings = [[0, 0]]
+    # no padding on the NC dimensions
+    tf_paddings = [[0, 0], [0, 0]]
+    # padding for the (D)HW dimensions
     for i in range(self.spatial_size):
       tf_paddings += [[pads[i * 2], pads[i * 2 + 1]]]
-    tf_paddings += [[0, 0]]
 
-    self.input = tf.pad(
-        self.input,
-        tf_paddings,
-        mode='CONSTANT',
-        constant_values=self.padding_constant)
+    self.input = tf.pad(self.input,
+                        tf_paddings,
+                        mode='CONSTANT',
+                        constant_values=self.padding_constant)
     # update input shape and pads values
     self.input_shape = tf_shape(self.input)
     self.pads = pads
@@ -513,10 +522,10 @@ class DilatedPooling(object):
   def _calc_argmax_without_padding(self, ind):
     """
             Calculate the original indices as they would be without padding
-        """
-    in_width = self.orig_input_shape[2]
-    padded_width = self.input_shape[2]
-    num_channels = self.input_shape[3]
+    """
+    in_width = self.orig_input_shape[3]
+    padded_width = self.input_shape[3]
+    num_channels = self.input_shape[1]
 
     # mod_floor op is not implemented on GPU
     # implement it using: a % b = a - (a // b) * b
@@ -535,21 +544,35 @@ class DilatedPooling(object):
   def dilated_maxpool_with_argmax(self, force_custom_impl=False):
     """
             Do a dilated maxpool and return indices/argmax
-        """
+    """
     # Tensorflow does not support maxpool_with_argmax on
     # spatial_size != 2
     assert self.spatial_size == 2
 
+    # tf.nn.max_pool_with_argmax only support data_format='NHWC'
+    self.compute_format = 'NHWC'
+    self.need_trans = self.storage_format != self.compute_format
+
     if list(self.dilations) != [1] * self.spatial_size or \
        force_custom_impl:
+
       # pad the input
       self._pad_input()
 
       new_input = self._remove_dilations()
       kernel_shape = [1] + list(self.kernel_shape) + [1]
-      pooled, new_ind = tf.nn.max_pool_with_argmax(
-          new_input, ksize=kernel_shape, strides=kernel_shape, padding="VALID")
+
+      if self.need_trans:
+        new_input = tf.transpose(new_input,
+                                 perm=get_perm_from_formats(
+                                     self.storage_format, self.compute_format))
+
+      pooled, new_ind = tf.nn.max_pool_with_argmax(new_input,
+                                                   ksize=kernel_shape,
+                                                   strides=kernel_shape,
+                                                   padding="VALID")
       new_ind = self._calc_orig_argmax(new_ind)
+
     else:
       self.pads = np.array([0] * self.spatial_size * 2)
       if type(self.padding) is list or \
@@ -565,20 +588,37 @@ class DilatedPooling(object):
 
       strides = [1] + list(self.strides) + [1]
       kernel_shape = [1] + list(self.kernel_shape) + [1]
-      pooled, new_ind = tf.nn.max_pool_with_argmax(
-          self.input, ksize=kernel_shape, strides=strides, padding=padding_)
+
+      if self.need_trans:
+        self.input = tf.transpose(self.input,
+                                  perm=get_perm_from_formats(
+                                      self.storage_format, self.compute_format))
+
+      pooled, new_ind = tf.nn.max_pool_with_argmax(self.input,
+                                                   ksize=kernel_shape,
+                                                   strides=strides,
+                                                   padding=padding_)
+
       # if there was padding, recalculate the returned index
       # to exclude the padding
-      if np.count_nonzero(self.pads) != 0:
-        new_ind = self._calc_argmax_without_padding(new_ind)
+      if self.is_known_shape:
+        if np.count_nonzero(self.pads) != 0:
+          new_ind = self._calc_argmax_without_padding(new_ind)
+      else:
+        new_ind = tf.where(tf.not_equal(tf.math.count_nonzero(self.pads), 0),
+                           self._calc_argmax_without_padding(new_ind), new_ind)
 
     return (pooled, new_ind)
 
-  def _lp_pool(self, input, ksize, strides, padding):
+  def _lp_pool(self, input, ksize, strides, padding, data_format):
     window_size = np.prod(ksize)
 
     input = tf.math.pow(tf.math.abs(input), self.p) * window_size
-    pooled = tf.nn.avg_pool(input, ksize=ksize, strides=strides, padding=padding)
+    pooled = tf.nn.avg_pool(input,
+                            ksize=ksize,
+                            strides=strides,
+                            padding=padding,
+                            data_format=data_format)
     pooled = tf.math.pow(pooled, 1.0 / self.p)
 
     return pooled
@@ -587,7 +627,7 @@ class DilatedPooling(object):
     """
             Does N-D dilated max/avg pooling. Pads the input if explicit or
             SAME_* padding is provided or ceil_mode is True
-        """
+    """
 
     assert self.is_supported()
 
@@ -611,29 +651,42 @@ class DilatedPooling(object):
       strides = [1] + list(self.strides) + [1]
       dilations = [1] + list(self.dilations) + [1]
 
+      # tf.nn.dilation2d only support data_format='NHWC'
+      self.compute_format = 'NHWC'
+      self.need_trans = self.storage_format.startswith("NC")
+      if self.need_trans:
+        self.input = tf.transpose(self.input,
+                                  perm=get_perm_from_formats(
+                                      self.storage_format, self.compute_format))
+
       filter = tf.zeros(
-          [self.kernel_shape[0], self.kernel_shape[1], self.input_shape[3]],
+          [self.kernel_shape[0], self.kernel_shape[1], self.input_shape[1]],
           self.input.dtype)
-      pooled = tf.nn.dilation2d(
-          input=self.input,
-          filters=filter,
-          strides=strides,
-          dilations=dilations,
-          padding=padding_,
-          data_format="NHWC")
+      pooled = tf.nn.dilation2d(input=self.input,
+                                filters=filter,
+                                strides=strides,
+                                dilations=dilations,
+                                padding=padding_,
+                                data_format="NHWC")
     # if spatial_size < 4 and strides == 1 or dilation == 1 use tf.nn.pool
     elif self.spatial_size < 4 and (self.strides == [1] * self.spatial_size or
             self.dilations == [1] * self.spatial_size) and \
             not force_custom_impl:
+
+      if self.need_trans:
+        self.input = tf.transpose(self.input,
+                                  perm=get_perm_from_formats(
+                                      self.storage_format, self.compute_format))
+
       # if strides == 1 and not LpPool use tf.nn.pool directly
       if self.strides == [1] * self.spatial_size and self.pooling_type != "LP":
-        pooled = tf.nn.pool(
-            self.input,
-            window_shape=self.kernel_shape,
-            dilations=self.dilations,
-            strides=self.strides,
-            padding=padding_,
-            pooling_type=self.pooling_type)
+        pooled = tf.nn.pool(self.input,
+                            window_shape=self.kernel_shape,
+                            dilations=self.dilations,
+                            strides=self.strides,
+                            padding=padding_,
+                            pooling_type=self.pooling_type,
+                            data_format=self.compute_format)
       else:
         # othwerwise check the pooling_type and use the correct op
         if self.pooling_type.startswith("MAX"):
@@ -645,8 +698,11 @@ class DilatedPooling(object):
         else:
           raise ValueError("%d-D %s pooling is not supported." %
                            (self.spatial_size, self.pooling_type))
-        pooled = op(self.input, ksize=self.kernel_shape, strides=self.strides,
-                    padding=padding_)
+        pooled = op(self.input,
+                    ksize=self.kernel_shape,
+                    strides=self.strides,
+                    padding=padding_,
+                    data_format=self.compute_format)
     # in any other case we use custom implementation _remove_dilations
     # to reduce atrous/dilated pooling into regular pooling and selecting
     # only the values of the input that should have been selected by
@@ -657,27 +713,34 @@ class DilatedPooling(object):
         # pad the input
         self._pad_input()
       input_ = self._remove_dilations()
-      if self.pooling_type=="LP":
-        pooled = self._lp_pool(
-            input_,
-            ksize=self.kernel_shape,
-            strides=self.kernel_shape,
-            padding="VALID")
+
+      if self.need_trans:
+        input_ = tf.transpose(input_,
+                              perm=get_perm_from_formats(
+                                  self.storage_format, self.compute_format))
+
+      if self.pooling_type == "LP":
+        pooled = self._lp_pool(input_,
+                               ksize=self.kernel_shape,
+                               strides=self.kernel_shape,
+                               padding="VALID",
+                               data_format=self.compute_format)
 
       else:
-        pooled = tf.nn.pool(
-            input_,
-            window_shape=self.kernel_shape,
-            strides=self.kernel_shape,
-            padding="VALID",
-            pooling_type=self.pooling_type)
+        pooled = tf.nn.pool(input_,
+                            window_shape=self.kernel_shape,
+                            strides=self.kernel_shape,
+                            padding="VALID",
+                            pooling_type=self.pooling_type,
+                            data_format=self.compute_format)
+
     return pooled
 
   def is_supported(self):
     """
             Function to check if the current set of arguments are
             supported for average pool
-        """
+    """
     # check for maxpool
     if self.pooling_type.startswith("MAX") or \
        self.pooling_type=="LP":
