@@ -1,5 +1,6 @@
 import tensorflow as tf
 
+from onnx_tf.common.tf_helper import tf_shape
 from onnx_tf.handlers.backend_handler import BackendHandler
 from onnx_tf.handlers.handler import onnx_op
 from onnx_tf.handlers.handler import tf_func
@@ -15,5 +16,5 @@ class RandomNormalLike(BackendHandler):
 
   @classmethod
   def version_1(cls, node, **kwargs):
-    inputs = [kwargs["tensor_dict"][node.inputs[0]].get_shape()]
+    inputs = [tf_shape(kwargs["tensor_dict"][node.inputs[0]])]
     return [cls.make_tensor_from_onnx_node(node, inputs=inputs, **kwargs)]
